@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::GenericInterface::Operation::Ticket::Common;
@@ -22,15 +22,9 @@ our $ObjectManagerDisabled = 1;
 
 Kernel::GenericInterface::Operation::Ticket::Common - Base class for all Ticket Operations
 
-=head1 SYNOPSIS
-
 =head1 PUBLIC INTERFACE
 
-=over 4
-
-=cut
-
-=item Init()
+=head2 Init()
 
 initialize the operation by checking the web service configuration and gather of the dynamic fields
 
@@ -90,7 +84,7 @@ sub Init {
     };
 }
 
-=item ValidateQueue()
+=head2 ValidateQueue()
 
 checks if the given queue or queue ID is valid.
 
@@ -154,7 +148,7 @@ sub ValidateQueue {
     return 1;
 }
 
-=item ValidateLock()
+=head2 ValidateLock()
 
 checks if the given lock or lock ID is valid.
 
@@ -204,7 +198,7 @@ sub ValidateLock {
     return 1;
 }
 
-=item ValidateType()
+=head2 ValidateType()
 
 checks if the given type or type ID is valid.
 
@@ -266,7 +260,7 @@ sub ValidateType {
     return 1;
 }
 
-=item ValidateCustomer()
+=head2 ValidateCustomer()
 
 checks if the given customer user or customer ID is valid.
 
@@ -325,7 +319,7 @@ sub ValidateCustomer {
     return 1;
 }
 
-=item ValidateService()
+=head2 ValidateService()
 
 checks if the given service or service ID is valid.
 
@@ -405,7 +399,7 @@ sub ValidateService {
     return 1;
 }
 
-=item ValidateSLA()
+=head2 ValidateSLA()
 
 checks if the given service or service ID is valid.
 
@@ -510,7 +504,7 @@ sub ValidateSLA {
     return 1;
 }
 
-=item ValidateState()
+=head2 ValidateState()
 
 checks if the given state or state ID is valid.
 
@@ -573,7 +567,7 @@ sub ValidateState {
     return 1;
 }
 
-=item ValidatePriority()
+=head2 ValidatePriority()
 
 checks if the given priority or priority ID is valid.
 
@@ -643,7 +637,7 @@ sub ValidatePriority {
     return 1;
 }
 
-=item ValidateOwner()
+=head2 ValidateOwner()
 
 checks if the given owner or owner ID is valid.
 
@@ -672,7 +666,7 @@ sub ValidateOwner {
     );
 }
 
-=item ValidateResponsible()
+=head2 ValidateResponsible()
 
 checks if the given responsible or responsible ID is valid.
 
@@ -701,7 +695,7 @@ sub ValidateResponsible {
     );
 }
 
-=item ValidatePendingTime()
+=head2 ValidatePendingTime()
 
 checks if the given pending time is valid.
 
@@ -751,17 +745,20 @@ sub ValidatePendingTime {
         return if int $Param{PendingTime}->{$TimeAttribute} < 0;
     }
 
-    # try to convert pending time to a SystemTime
-    my $SystemTime = $Kernel::OM->Get('Kernel::System::Time')->Date2SystemTime(
-        %{ $Param{PendingTime} },
-        Second => 0,
+    # try to convert pending time to a DateTime object
+    my $PendingTime = $Kernel::OM->Create(
+        'Kernel::System::DateTime',
+        ObjectParams => {
+            %{ $Param{PendingTime} },
+            Second => 0,
+            }
     );
-    return if !$SystemTime;
+    return if !$PendingTime;
 
     return 1;
 }
 
-=item ValidateAutoResponseType()
+=head2 ValidateAutoResponseType()
 
 checks if the given AutoResponseType is valid.
 
@@ -791,76 +788,7 @@ sub ValidateAutoResponseType {
     return;
 }
 
-=item ValidateArticleType()
-
-checks if the given ArticleType or ArticleType ID is valid.
-
-    my $Success = $CommonObject->ValidateArticleType(
-        ArticleTypeID => 123,
-    );
-
-    my $Success = $CommonObject->ValidateArticleType(
-        ArticleType => 'some ArticleType',
-    );
-
-    returns
-    $Success = 1            # or 0
-
-=cut
-
-sub ValidateArticleType {
-    my ( $Self, %Param ) = @_;
-
-    # check needed stuff
-    return if !$Param{ArticleTypeID} && !$Param{ArticleType};
-
-    # get ticket object
-    my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
-
-    my %ArticleTypeList = $TicketObject->ArticleTypeList(
-        Result => 'HASH',
-
-        # add type parameter for customer as requester with UserType parameter, if is not set
-        # to  'Customer' the Type parameter is ignored
-        Type => $Param{UserType} || '',
-    );
-
-    # check for ArticleType name sent
-    if (
-        $Param{ArticleType}
-        && $Param{ArticleType} ne ''
-        && !$Param{ArticleTypeID}
-        )
-    {
-        my $ArticleTypeID = $TicketObject->ArticleTypeLookup(
-            ArticleType => $Param{ArticleType},
-        );
-
-        return if !$ArticleTypeID;
-
-        # check if $ArticleType is valid
-        return if !$ArticleTypeList{$ArticleTypeID};
-    }
-
-    # otherwise use ArticleTypeID
-    elsif ( $Param{ArticleTypeID} ) {
-        my $ArticleType = $TicketObject->ArticleTypeLookup(
-            ArticleTypeID => $Param{ArticleTypeID},
-        );
-
-        return if !$ArticleType;
-
-        # check if $ArticleType is valid
-        return if !$ArticleTypeList{ $Param{ArticleTypeID} };
-    }
-    else {
-        return;
-    }
-
-    return 1;
-}
-
-=item ValidateFrom()
+=head2 ValidateFrom()
 
 checks if the given from is valid.
 
@@ -892,7 +820,43 @@ sub ValidateFrom {
     return 1;
 }
 
-=item ValidateSenderType()
+=head2 ValidateArticleCommunicationChannel()
+
+checks if provided Communication Channel is valid.
+
+    my $Success = $CommonObject->ValidateArticleCommunicationChannel(
+        CommunicationChannel   => 'Internal',   # optional
+                                                # or
+        CommunicationChannelID => 1,            # optional
+    );
+
+    returns
+    $Success = 1            # or 0
+
+=cut
+
+sub ValidateArticleCommunicationChannel {
+    my ( $Self, %Param ) = @_;
+
+    # check needed stuff
+    if ( !$Param{CommunicationChannel} && !$Param{CommunicationChannelID} ) {
+        return;
+    }
+
+    my %CommunicationChannel = $Kernel::OM->Get('Kernel::System::CommunicationChannel')->ChannelGet(
+        ChannelID   => $Param{CommunicationChannelID},
+        ChannelName => $Param{CommunicationChannel},
+    );
+
+    return if !%CommunicationChannel;
+
+    # TicketCreate and TicketUpdate operations should only work with MIME based communication channels
+    return if $CommunicationChannel{ChannelName} !~ m{\AEmail|Internal|Phone\z}msxi;
+
+    return 1;
+}
+
+=head2 ValidateSenderType()
 
 checks if the given SenderType or SenderType ID is valid.
 
@@ -915,12 +879,9 @@ sub ValidateSenderType {
     # check needed stuff
     return if !$Param{SenderTypeID} && !$Param{SenderType};
 
-    # get ticket object
-    my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
+    my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
 
-    my %SenderTypeList = $TicketObject->ArticleSenderTypeList(
-        Result => 'HASH',
-    );
+    my %SenderTypeList = $ArticleObject->ArticleSenderTypeList();
 
     # check for SenderType name sent
     if (
@@ -929,7 +890,7 @@ sub ValidateSenderType {
         && !$Param{SenderTypeID}
         )
     {
-        my $SenderTypeID = $TicketObject->ArticleSenderTypeLookup(
+        my $SenderTypeID = $ArticleObject->ArticleSenderTypeLookup(
             SenderType => $Param{SenderType},
         );
 
@@ -941,7 +902,7 @@ sub ValidateSenderType {
 
     # otherwise use SenderTypeID
     elsif ( $Param{SenderTypeID} ) {
-        my $SenderType = $TicketObject->ArticleSenderTypeLookup(
+        my $SenderType = $ArticleObject->ArticleSenderTypeLookup(
             SenderTypeID => $Param{SenderTypeID},
         );
 
@@ -957,7 +918,7 @@ sub ValidateSenderType {
     return 1;
 }
 
-=item ValidateMimeType()
+=head2 ValidateMimeType()
 
 checks if the given MimeType is valid.
 
@@ -981,7 +942,7 @@ sub ValidateMimeType {
     return 1;
 }
 
-=item ValidateCharset()
+=head2 ValidateCharset()
 
 checks if the given Charset is valid.
 
@@ -1006,7 +967,7 @@ sub ValidateCharset {
     return 1;
 }
 
-=item ValidateHistoryType()
+=head2 ValidateHistoryType()
 
 checks if the given HistoryType is valid.
 
@@ -1043,7 +1004,7 @@ sub ValidateHistoryType {
     return 1;
 }
 
-=item ValidateTimeUnit()
+=head2 ValidateTimeUnit()
 
 checks if the given TimeUnit is valid.
 
@@ -1068,7 +1029,7 @@ sub ValidateTimeUnit {
     return 1;
 }
 
-=item ValidateUserID()
+=head2 ValidateUserID()
 
 checks if the given user ID is valid.
 
@@ -1092,7 +1053,7 @@ sub ValidateUserID {
     );
 }
 
-=item ValidateDynamicFieldName()
+=head2 ValidateDynamicFieldName()
 
 checks if the given dynamic field name is valid.
 
@@ -1118,7 +1079,7 @@ sub ValidateDynamicFieldName {
     return 1;
 }
 
-=item ValidateDynamicFieldValue()
+=head2 ValidateDynamicFieldValue()
 
 checks if the given dynamic field value is valid.
 
@@ -1153,17 +1114,53 @@ sub ValidateDynamicFieldValue {
     # get dynamic field config
     my $DynamicFieldConfig = $Self->{DynamicFieldLookup}->{ $Param{Name} };
 
-    # Validate value.
-    my $ValidateValue = $Kernel::OM->Get('Kernel::System::DynamicField::Backend')->FieldValueValidate(
+    my $ValueType = $Kernel::OM->Get('Kernel::System::DynamicField::Backend')->ValueValidate(
         DynamicFieldConfig => $DynamicFieldConfig,
         Value              => $Param{Value},
         UserID             => 1,
     );
 
-    return $ValidateValue;
+    return if !$ValueType;
+
+    # Check if value parameter exists in config of possible values, for example for dropdown/multi-select fields.
+    #   Please see bug#13444 for more information.
+    if (
+        defined $Param{Value}
+        && length $Param{Value}
+        && (
+            IsArrayRefWithData( $DynamicFieldConfig->{Config}->{PossibleValues} )
+            || IsHashRefWithData( $DynamicFieldConfig->{Config}->{PossibleValues} )
+        )
+        )
+    {
+        my @Values;
+        if ( ref $Param{Value} eq 'ARRAY' ) {
+            @Values = @{ $Param{Value} };
+        }
+        else {
+            push @Values, $Param{Value};
+        }
+
+        if ( IsArrayRefWithData( $DynamicFieldConfig->{Config}->{PossibleValues} ) ) {
+            for my $Value (@Values) {
+                if ( !grep { $_ eq $Value } @{ $DynamicFieldConfig->{Config}->{PossibleValues} } ) {
+                    return;
+                }
+            }
+        }
+        else {
+            for my $Value (@Values) {
+                if ( !grep { $_ eq $Value } keys %{ $DynamicFieldConfig->{Config}->{PossibleValues} } ) {
+                    return;
+                }
+            }
+        }
+    }
+
+    return 1;
 }
 
-=item ValidateDynamicFieldObjectType()
+=head2 ValidateDynamicFieldObjectType()
 
 checks if the given dynamic field name is valid.
 
@@ -1193,7 +1190,7 @@ sub ValidateDynamicFieldObjectType {
     return 1;
 }
 
-=item SetDynamicFieldValue()
+=head2 SetDynamicFieldValue()
 
 sets the value of a dynamic field.
 
@@ -1202,7 +1199,7 @@ sets the value of a dynamic field.
         Value     => 'some value',          # String or Integer or DateTime format
         TicketID  => 123
         ArticleID => 123
-        UserID => 123,
+        UserID    => 123,
     );
 
     my $Result = $CommonObject->SetDynamicFieldValue(
@@ -1279,16 +1276,15 @@ sub SetDynamicFieldValue {
     };
 }
 
-=item CreateAttachment()
+=head2 CreateAttachment()
 
 creates a new attachment for the given article.
 
     my $Result = $CommonObject->CreateAttachment(
-        Content     => $Data,                   # file content (Base64 encoded)
-        ContentType => 'some content type',
-        Filename    => 'some filename',
-        ArticleID   => 456,
-        UserID      => 123.
+        TicketID   => 123,
+        Attachment => $Data,                   # file content (Base64 encoded)
+        ArticleID  => 456,
+        UserID     => 123,
     );
 
     returns
@@ -1307,7 +1303,7 @@ sub CreateAttachment {
     my ( $Self, %Param ) = @_;
 
     # check needed stuff
-    for my $Needed (qw(Attachment ArticleID UserID)) {
+    for my $Needed (qw(TicketID Attachment ArticleID UserID)) {
         if ( !$Param{$Needed} ) {
             return {
                 Success      => 0,
@@ -1316,8 +1312,13 @@ sub CreateAttachment {
         }
     }
 
+    my $ArticleBackendObject = $Kernel::OM->Get('Kernel::System::Ticket::Article')->BackendForArticle(
+        TicketID  => $Param{TicketID},
+        ArticleID => $Param{ArticleID},
+    );
+
     # write attachment
-    my $Success = $Kernel::OM->Get('Kernel::System::Ticket')->ArticleWriteAttachment(
+    my $Success = $ArticleBackendObject->ArticleWriteAttachment(
         %{ $Param{Attachment} },
         Content   => MIME::Base64::decode_base64( $Param{Attachment}->{Content} ),
         ArticleID => $Param{ArticleID},
@@ -1329,7 +1330,7 @@ sub CreateAttachment {
     };
 }
 
-=item CheckCreatePermissions ()
+=head2 CheckCreatePermissions ()
 
 Tests if the user have the permissions to create a ticket on a determined queue
 
@@ -1385,7 +1386,7 @@ sub CheckCreatePermissions {
     return 1;
 }
 
-=item CheckAccessPermissions()
+=head2 CheckAccessPermissions()
 
 Tests if the user have access permissions over a ticket
 
@@ -1426,7 +1427,7 @@ sub CheckAccessPermissions {
 
 =begin Internal:
 
-=item _ValidateUser()
+=head2 _ValidateUser()
 
 checks if the given user or user ID is valid.
 
@@ -1481,57 +1482,16 @@ sub _ValidateUser {
     return 1;
 }
 
-=item _CharsetList()
-
-DEPRECATED: This function will be removed in further versions of OTRS.
-
-returns a list of all available charsets.
-
-    my $CharsetList = $CommonObject->_CharsetList(
-        UserID => 123,
-    );
-
-    returns
-    $Success = {
-        #...
-        iso-8859-1  => 1,
-        iso-8859-15 => 1,
-        MacRoman    => 1,
-        utf8        => 1,
-        #...
-    }
-
-=cut
-
-sub _CharsetList {
-    my ( $Self, %Param ) = @_;
-
-    # get charset array
-    use Encode;
-    my @CharsetList = Encode->encodings(":all");
-
-    my %CharsetHash;
-
-    # create a charset lookup table
-    for my $Charset (@CharsetList) {
-        $CharsetHash{$Charset} = 1;
-    }
-
-    return \%CharsetHash;
-}
-
 1;
 
 =end Internal:
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
