@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2020 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::GenericAgent;
@@ -18,6 +18,7 @@ use Kernel::System::VariableCheck qw(:all);
 our @ObjectDependencies = (
     'Kernel::Config',
     'Kernel::System::Cache',
+    'Kernel::System::DateTime',
     'Kernel::System::DB',
     'Kernel::System::DynamicField',
     'Kernel::System::DynamicField::Backend',
@@ -26,29 +27,25 @@ our @ObjectDependencies = (
     'Kernel::System::Queue',
     'Kernel::System::State',
     'Kernel::System::Ticket',
-    'Kernel::System::Time',
+    'Kernel::System::Ticket::Article',
+    'Kernel::System::TemplateGenerator',
+    'Kernel::System::CustomerUser',
 );
 
 =head1 NAME
 
 Kernel::System::GenericAgent - to manage the generic agent jobs
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 All functions to manage the generic agent and the generic agent jobs.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
+=head2 new()
 
-=cut
+Don't use the constructor directly, use the ObjectManager instead:
 
-=item new()
-
-create an object. Do not use it directly, instead use:
-
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new();
     my $GenericAgentObject = $Kernel::OM->Get('Kernel::System::GenericAgent');
 
 =cut
@@ -79,11 +76,11 @@ sub new {
     my %Map = (
         TicketNumber            => 'SCALAR',
         Title                   => 'SCALAR',
-        From                    => 'SCALAR',
-        To                      => 'SCALAR',
-        Cc                      => 'SCALAR',
-        Subject                 => 'SCALAR',
-        Body                    => 'SCALAR',
+        MIMEBase_From           => 'SCALAR',
+        MIMEBase_To             => 'SCALAR',
+        MIMEBase_Cc             => 'SCALAR',
+        MIMEBase_Subject        => 'SCALAR',
+        MIMEBase_Body           => 'SCALAR',
         TimeUnit                => 'SCALAR',
         CustomerID              => 'SCALAR',
         CustomerUserLogin       => 'SCALAR',
@@ -164,7 +161,7 @@ sub new {
     return $Self;
 }
 
-=item JobRun()
+=head2 JobRun()
 
 run a generic agent job
 
@@ -507,7 +504,7 @@ sub JobRun {
     return 1;
 }
 
-=item JobList()
+=head2 JobList()
 
 returns a hash of jobs
 
@@ -551,7 +548,7 @@ sub JobList {
     return %Data;
 }
 
-=item JobGet()
+=head2 JobGet()
 
 returns a hash of the job data
 
@@ -739,7 +736,7 @@ sub JobGet {
     return %Data;
 }
 
-=item JobAdd()
+=head2 JobAdd()
 
 adds a new job to the database
 
@@ -774,7 +771,7 @@ sub JobAdd {
     if (%Check) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message  => "Can't add job '$Param{Name}', job already exists!",
+            Message  => "A job with the name '$Param{Name}' already exists.",
         );
         return;
     }
@@ -818,7 +815,7 @@ sub JobAdd {
     return 1;
 }
 
-=item JobDelete()
+=head2 JobDelete()
 
 deletes a job from the database
 
@@ -865,7 +862,7 @@ sub JobDelete {
     return 1;
 }
 
-=item JobEventList()
+=head2 JobEventList()
 
 returns a hash of events for each job
 
@@ -910,7 +907,7 @@ sub JobEventList {
 
 =cut
 
-=item _JobRunTicket()
+=head2 _JobRunTicket()
 
 run a generic agent job on a ticket
 
@@ -972,23 +969,69 @@ sub _JobRunTicket {
         );
     }
 
+    my $ContentType = 'text/plain';
+
     # add note if wanted
     if ( $Param{Config}->{New}->{Note}->{Body} || $Param{Config}->{New}->{NoteBody} ) {
         if ( $Self->{NoticeSTDOUT} ) {
             print "  - Add note to Ticket $Ticket\n";
         }
-        my $ArticleID = $TicketObject->ArticleCreate(
-            TicketID    => $Param{TicketID},
-            ArticleType => $Param{Config}->{New}->{Note}->{ArticleType} || 'note-internal',
-            SenderType  => 'agent',
-            From        => $Param{Config}->{New}->{Note}->{From}
+
+        my %Ticket = $TicketObject->TicketGet(
+            TicketID      => $Param{TicketID},
+            DynamicFields => 0,
+        );
+
+        my %CustomerUserData;
+
+        # We can only do OTRS Tag replacement if we have a CustomerUserID (langauge settings...)
+        if ( IsHashRefWithData( \%Ticket ) && IsStringWithData( $Ticket{CustomerUserID} ) ) {
+            my %CustomerUserData = $Kernel::OM->Get('Kernel::System::CustomerUser')->CustomerUserDataGet(
+                User => $Ticket{CustomerUserID},
+            );
+
+            my %Notification = (
+                Subject     => $Param{Config}->{New}->{NoteSubject},
+                Body        => $Param{Config}->{New}->{NoteBody},
+                ContentType => 'text/plain',
+            );
+
+            my %GenericAgentArticle = $Kernel::OM->Get('Kernel::System::TemplateGenerator')->GenericAgentArticle(
+                TicketID     => $Param{TicketID},
+                Recipient    => \%CustomerUserData,    # Agent or Customer data get result
+                Notification => \%Notification,
+                UserID       => $Param{UserID},
+            );
+
+            if (
+                IsStringWithData( $GenericAgentArticle{Body} )
+                || IsHashRefWithData( $GenericAgentArticle{Subject} )
+                )
+            {
+                $Param{Config}->{New}->{Note}->{Subject} = $GenericAgentArticle{Subject} || '';
+                $Param{Config}->{New}->{Note}->{Body}    = $GenericAgentArticle{Body}    || '';
+                $ContentType                             = $GenericAgentArticle{ContentType};
+            }
+        }
+
+        my $ArticleBackendObject = $Kernel::OM->Get('Kernel::System::Ticket::Article')->BackendForChannel(
+            ChannelName => 'Internal',
+        );
+
+        my $ArticleID = $ArticleBackendObject->ArticleCreate(
+            TicketID             => $Param{TicketID},
+            SenderType           => 'agent',
+            IsVisibleForCustomer => $Param{Config}->{New}->{Note}->{IsVisibleForCustomer}
+                // $Param{Config}->{New}->{NoteIsVisibleForCustomer}
+                // 0,
+            From => $Param{Config}->{New}->{Note}->{From}
                 || $Param{Config}->{New}->{NoteFrom}
                 || 'GenericAgent',
             Subject => $Param{Config}->{New}->{Note}->{Subject}
                 || $Param{Config}->{New}->{NoteSubject}
                 || 'Note',
-            Body           => $Param{Config}->{New}->{Note}->{Body} || $Param{Config}->{New}->{NoteBody},
-            MimeType       => 'text/plain',
+            Body => $Param{Config}->{New}->{Note}->{Body} || $Param{Config}->{New}->{NoteBody},
+            MimeType       => $ContentType,
             Charset        => 'utf-8',
             UserID         => $Param{UserID},
             HistoryType    => 'AddNote',
@@ -1069,21 +1112,16 @@ sub _JobRunTicket {
         }
 
         # add systemtime
-        $PendingTime += $Kernel::OM->Get('Kernel::System::Time')->SystemTime();
-
-        # get date
-        my ( $Sec, $Min, $Hour, $Day, $Month, $Year, $WeekDay )
-            = $Kernel::OM->Get('Kernel::System::Time')->SystemTime2Date(
-            SystemTime => $PendingTime,
-            );
+        my $DateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
+        $DateTimeObject->Add( Seconds => $PendingTime );
 
         # set pending time
         $TicketObject->TicketPendingTimeSet(
-            Year     => $Year,
-            Month    => $Month,
-            Day      => $Day,
-            Hour     => $Hour,
-            Minute   => $Min,
+            Year     => $DateTimeObject->Format( Format => '%Y' ),
+            Month    => $DateTimeObject->Format( Format => '%m' ),
+            Day      => $DateTimeObject->Format( Format => '%d' ),
+            Hour     => $DateTimeObject->Format( Format => '%H' ),
+            Minute   => $DateTimeObject->Format( Format => '%M' ),
             TicketID => $Param{TicketID},
             UserID   => $Param{UserID},
         );
@@ -1290,7 +1328,7 @@ sub _JobRunTicket {
 
         next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
 
-        # extract the dynamic field value form the web request
+        # extract the dynamic field value from the web request
         my $Value = $DynamicFieldBackendObject->EditFieldValueGet(
             DynamicFieldConfig => $DynamicFieldConfig,
             Template           => $Param{Config}->{New},
@@ -1336,10 +1374,7 @@ sub _JobRunTicket {
     }
 
     # run module
-    my $AllowCustomModuleExecution
-        = $Kernel::OM->Get('Kernel::Config')->Get('Ticket::GenericAgentAllowCustomModuleExecution') || 0;
-
-    if ( $Param{Config}->{New}->{Module} && $AllowCustomModuleExecution ) {
+    if ( $Param{Config}->{New}->{Module} ) {
         if ( $Self->{NoticeSTDOUT} ) {
             print "  - Use module ($Param{Config}->{New}->{Module}) for Ticket $Ticket.\n";
         }
@@ -1378,15 +1413,6 @@ sub _JobRunTicket {
             }
         }
     }
-    elsif ( $Param{Config}->{New}->{Module} && !$AllowCustomModuleExecution ) {
-        if ( $Self->{NoticeSTDOUT} ) {
-            print "  - Use module ($Param{Config}->{New}->{Module}) is not allowed by the system configuration.\n";
-        }
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Use module ($Param{Config}->{New}->{Module}) is not allowed by the system configuration.",
-        );
-    }
 
     # set new archive flag
     if (
@@ -1406,10 +1432,7 @@ sub _JobRunTicket {
     }
 
     # cmd
-    my $AllowCustomScriptExecution
-        = $Kernel::OM->Get('Kernel::Config')->Get('Ticket::GenericAgentAllowCustomScriptExecution') || 0;
-
-    if ( $Param{Config}->{New}->{CMD} && $AllowCustomScriptExecution ) {
+    if ( $Param{Config}->{New}->{CMD} ) {
         if ( $Self->{NoticeSTDOUT} ) {
             print "  - Execute '$Param{Config}->{New}->{CMD}' for Ticket $Ticket.\n";
         }
@@ -1426,15 +1449,6 @@ sub _JobRunTicket {
             );
         }
     }
-    elsif ( $Param{Config}->{New}->{CMD} && !$AllowCustomScriptExecution ) {
-        if ( $Self->{NoticeSTDOUT} ) {
-            print "  - Execute '$Param{Config}->{New}->{CMD}' is not allowed by the system configuration..\n";
-        }
-        $Kernel::OM->Get('Kernel::System::Log')->Log(
-            Priority => 'error',
-            Message  => "Execute '$Param{Config}->{New}->{CMD}' is not allowed by the system configuration..",
-        );
-    }
 
     # delete ticket
     if ( $Param{Config}->{New}->{Delete} ) {
@@ -1470,31 +1484,19 @@ sub _JobUpdateRunTime {
     # get database object
     my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
 
-    # check if job name already exists
-    return if !$DBObject->Prepare(
-        SQL  => 'SELECT job_key, job_value FROM generic_agent_jobs WHERE job_name = ?',
-        Bind => [ \$Param{Name} ],
+    # delete old run times
+    return if !$DBObject->Do(
+        SQL  => 'DELETE FROM generic_agent_jobs WHERE job_name = ? AND job_key IN (?, ?)',
+        Bind => [ \$Param{Name}, \'ScheduleLastRun', \'ScheduleLastRunUnixTime' ],
     );
-    my @Data;
-    while ( my @Row = $DBObject->FetchrowArray() ) {
-        if ( $Row[0] =~ /^(ScheduleLastRun|ScheduleLastRunUnixTime)/ ) {
-            push @Data,
-                {
-                Key   => $Row[0],
-                Value => $Row[1]
-                };
-        }
-    }
 
     # get time object
-    my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+    my $DateTimeObject = $Kernel::OM->Create('Kernel::System::DateTime');
 
     # update new run time
     my %Insert = (
-        ScheduleLastRun => $TimeObject->SystemTime2TimeStamp(
-            SystemTime => $TimeObject->SystemTime()
-        ),
-        ScheduleLastRunUnixTime => $TimeObject->SystemTime(),
+        ScheduleLastRun         => $DateTimeObject->ToString(),
+        ScheduleLastRunUnixTime => $DateTimeObject->ToEpoch(),
     );
 
     for my $Key ( sort keys %Insert ) {
@@ -1504,15 +1506,6 @@ sub _JobUpdateRunTime {
         );
     }
 
-    # remove old times
-    for my $Time (@Data) {
-        $DBObject->Do(
-            SQL => 'DELETE FROM generic_agent_jobs WHERE '
-                . 'job_name = ? AND job_key = ? AND job_value = ?',
-            Bind => [ \$Param{Name}, \$Time->{Key}, \$Time->{Value} ],
-        );
-    }
-
     $Kernel::OM->Get('Kernel::System::Cache')->Delete(
         Key  => 'JobGet::' . $Param{Name},
         Type => 'GenericAgent',
@@ -1525,14 +1518,12 @@ sub _JobUpdateRunTime {
 
 =end Internal:
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
