@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::ProcessManagement::TransitionAction::TicketCreate;
@@ -14,7 +14,7 @@ use utf8;
 
 use Kernel::System::VariableCheck qw(:all);
 
-use base qw(Kernel::System::ProcessManagement::TransitionAction::Base);
+use parent qw(Kernel::System::ProcessManagement::TransitionAction::Base);
 
 our @ObjectDependencies = (
     'Kernel::Config',
@@ -24,7 +24,8 @@ our @ObjectDependencies = (
     'Kernel::System::Log',
     'Kernel::System::State',
     'Kernel::System::Ticket',
-    'Kernel::System::Time',
+    'Kernel::System::Ticket::Article',
+    'Kernel::System::DateTime',
     'Kernel::System::User',
 );
 
@@ -32,22 +33,16 @@ our @ObjectDependencies = (
 
 Kernel::System::ProcessManagement::TransitionAction::TicketCreate - A module to create a ticket
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
-All TicketArticleCreate functions.
+All TicketCreate functions.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
+=head2 new()
 
-=cut
+Don't use the constructor directly, use the ObjectManager instead:
 
-=item new()
-
-create an object. Do not use it directly, instead use:
-
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new();
     my $TicketCreateObject = $Kernel::OM->Get('Kernel::System::ProcessManagement::TransitionAction::TicketCreate');
 
 =cut
@@ -62,11 +57,11 @@ sub new {
     return $Self;
 }
 
-=item Run()
+=head2 Run()
 
     Run Data
 
-    my $TicketArticleCreateResult = $TicketArticleCreateActionObject->Run(
+    my $TicketCreateResult = $TicketCreateActionObject->Run(
         UserID                   => 123,
         Ticket                   => \%Ticket,   # required
         ProcessEntityID          => 'P123',
@@ -95,34 +90,14 @@ sub new {
             PendingTimeDiff => 123 ,                  # optional (for pending states)
 
             # article required: (if one of them is not present, article will not be created without any error message)
-            ArticleType      => 'note-internal',                        # note-external|phone|fax|sms|...
-                                                                        #   excluding any email type
-            SenderType       => 'agent',                                # agent|system|customer
-            ContentType      => 'text/plain; charset=ISO-8859-15',      # or optional Charset & MimeType
-            Subject          => 'some short description',               # required
-            Body             => 'the message text',                     # required
-            HistoryType      => 'OwnerUpdate',                          # EmailCustomer|Move|AddNote|PriorityUpdate|WebRequestCustomer|...
-            HistoryComment   => 'Some free text!',
+            SenderType           => 'agent',                            # agent|system|customer
+            IsVisibleForCustomer => 1,                                  # required
+            CommunicationChannel => 'Internal',                         # Internal|Phone|Email|..., default: Internal
+
+            %DataPayload,                                               # some parameters depending of each communication channel
 
             # article optional:
-            From             => 'Some Agent <email@example.com>',       # not required but useful
-            To               => 'Some Customer A <customer-a@example.com>', # not required but useful
-            Cc               => 'Some Customer B <customer-b@example.com>', # not required but useful
-            ReplyTo          => 'Some Customer B <customer-b@example.com>', # not required
-            MessageID        => '<asdasdasd.123@example.com>',          # not required but useful
-            InReplyTo        => '<asdasdasd.12@example.com>',           # not required but useful
-            References       => '<asdasdasd.1@example.com> <asdasdasd.12@example.com>', # not required but useful
-            NoAgentNotify    => 0,                                      # if you don't want to send agent notifications
-            AutoResponseType => 'auto reply'                            # auto reject|auto follow up|auto reply/new ticket|auto remove
-
-            ForceNotificationToUserID   => [ 1, 43, 56 ],               # if you want to force somebody
-            ExcludeNotificationToUserID => [ 43,56 ],                   # if you want full exclude somebody from notifications,
-                                                                        # will also be removed in To: line of article,
-                                                                        # higher prio as ForceNotificationToUserID
-            ExcludeMuteNotificationToUserID => [ 43,56 ],               # the same as ExcludeNotificationToUserID but only the
-                                                                        # sending gets muted, agent will still shown in To:
-                                                                        # line of article
-            TimeUnit                        => 123
+            TimeUnit => 123
 
             # other:
             DynamicField_NameX => $Value,
@@ -160,6 +135,7 @@ sub Run {
 
     # use ticket attributes if needed
     $Self->_ReplaceTicketAttributes(%Param);
+    $Self->_ReplaceAdditionalAttributes(%Param);
 
     # convert scalar items into array references
     for my $Attribute (
@@ -251,18 +227,14 @@ sub Run {
 
         if ( $Param{Config}->{PendingTime} ) {
 
-            # get time object
-            my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
-
-            # convert pending time to system time
-            my $SystemTime = $TimeObject->TimeStamp2SystemTime(
-                String => $Param{Config}->{PendingTime},
-            );
-
-            # convert it back again so we are sure the sting is correct
-            my $TimeStamp = $TimeObject->SystemTime2TimeStamp(
-                SystemTime => $SystemTime,
+            # get datetime object
+            my $DateTimeObject = $Kernel::OM->Create(
+                'Kernel::System::DateTime',
+                ObjectParams => {
+                    String => $Param{Config}->{PendingTime}
+                    }
             );
+            my $TimeStamp = $DateTimeObject->ToString();
 
             # set pending time
             $TicketObject->TicketPendingTimeSet(
@@ -282,25 +254,12 @@ sub Run {
         }
     }
 
-    # extract the article params
-    my %ArticleParam;
-    for my $Attribute (
-        qw( ArticleType SenderType ContentType Subject Body HistoryType
-        HistoryComment From To Cc ReplyTo MessageID InReplyTo References NoAgentNotify
-        AutoResponseType ForceNotificationToUserID ExcludeNotificationToUserID
-        ExcludeMuteNotificationToUserID
-        )
-        )
-    {
-        if ( defined $Param{Config}->{$Attribute} ) {
-            $ArticleParam{$Attribute} = $Param{Config}->{$Attribute};
-        }
-    }
+    $Param{Config}->{CommunicationChannel} ||= 'Internal';
 
     # check if article can be created
     my $ArticleCreate = 1;
-    for my $Needed (qw(ArticleType SenderType ContentType Subject Body HistoryType HistoryComment)) {
-        if ( !$ArticleParam{$Needed} ) {
+    for my $Needed (qw(SenderType IsVisibleForCustomer)) {
+        if ( !defined $Param{Config}->{$Needed} ) {
             $ArticleCreate = 0;
         }
     }
@@ -309,47 +268,36 @@ sub Run {
 
     if ($ArticleCreate) {
 
-        my $ValidArticleType = 1;
+        my $ArticleBackendObject = $Kernel::OM->Get('Kernel::System::Ticket::Article')->BackendForChannel(
+            ChannelName => $Param{Config}->{CommunicationChannel},
+        );
 
-        # check ArticleType
-        if ( $ArticleParam{ArticleType} =~ m{\A email }msxi ) {
+        # Create article for the new ticket.
+        $ArticleID = $ArticleBackendObject->ArticleCreate(
+            %{ $Param{Config} },
+            TicketID => $TicketID,
+            UserID   => $Param{UserID},
+        );
+
+        if ( !$ArticleID ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => $CommonMessage
-                    . "ArticleType $Param{Config}->{ArticleType} is not supported",
+                    . "Couldn't create Article on Ticket: $TicketID from Ticket: "
+                    . $Param{Ticket}->{TicketID} . '!',
             );
-            $ValidArticleType = 0;
         }
+        else {
 
-        if ($ValidArticleType) {
-
-            # create article for the new ticket
-            $ArticleID = $TicketObject->ArticleCreate(
-                %ArticleParam,
-                TicketID => $TicketID,
-                UserID   => $Param{UserID},
-            );
-
-            if ( !$ArticleID ) {
-                $Kernel::OM->Get('Kernel::System::Log')->Log(
-                    Priority => 'error',
-                    Message  => $CommonMessage
-                        . "Couldn't create Article on Ticket: $TicketID from Ticket: "
-                        . $Param{Ticket}->{TicketID} . '!',
+            # set time units
+            if ( $Param{Config}->{TimeUnit} ) {
+                $TicketObject->TicketAccountTime(
+                    TicketID  => $TicketID,
+                    ArticleID => $ArticleID,
+                    TimeUnit  => $Param{Config}->{TimeUnit},
+                    UserID    => $Param{UserID},
                 );
             }
-            else {
-
-                # set time units
-                if ( $Param{Config}->{TimeUnit} ) {
-                    $TicketObject->TicketAccountTime(
-                        TicketID  => $TicketID,
-                        ArticleID => $ArticleID,
-                        TimeUnit  => $Param{Config}->{TimeUnit},
-                        UserID    => $Param{UserID},
-                    );
-                }
-            }
         }
     }
 
@@ -480,14 +428,12 @@ sub Run {
 
 1;
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
