@@ -1,9 +1,9 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (GPL). If you
-# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
+# the enclosed file COPYING for license information (AGPL). If you
+# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
 # --
 
 package Kernel::System::StandardTemplate;
@@ -22,22 +22,16 @@ our @ObjectDependencies = (
 
 Kernel::System::StandardTemplate - standard template lib
 
-=head1 SYNOPSIS
+=head1 DESCRIPTION
 
 All standard template functions. E. g. to add standard template or other functions.
 
 =head1 PUBLIC INTERFACE
 
-=over 4
-
-=cut
-
-=item new()
+=head2 new()
 
 create an object
 
-    use Kernel::System::ObjectManager;
-    local $Kernel::OM = Kernel::System::ObjectManager->new();
     my $StandardTemplateObject = $Kernel::OM->Get('Kernel::System::StandardTemplate');
 
 =cut
@@ -52,7 +46,7 @@ sub new {
     return $Self;
 }
 
-=item StandardTemplateAdd()
+=head2 StandardTemplateAdd()
 
 add new standard template
 
@@ -85,7 +79,7 @@ sub StandardTemplateAdd {
     if ( $Self->NameExistsCheck( Name => $Param{Name} ) ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message  => "A standard template with name '$Param{Name}' already exists!"
+            Message  => "A standard template with the name '$Param{Name}' already exists.",
         );
         return;
     }
@@ -123,7 +117,7 @@ sub StandardTemplateAdd {
     return $ID;
 }
 
-=item StandardTemplateGet()
+=head2 StandardTemplateGet()
 
 get standard template attributes
 
@@ -194,7 +188,7 @@ sub StandardTemplateGet {
     return %Data;
 }
 
-=item StandardTemplateDelete()
+=head2 StandardTemplateDelete()
 
 delete a standard template
 
@@ -245,7 +239,7 @@ sub StandardTemplateDelete {
     return 1;
 }
 
-=item StandardTemplateUpdate()
+=head2 StandardTemplateUpdate()
 
 update standard template attributes
 
@@ -285,7 +279,7 @@ sub StandardTemplateUpdate {
     {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message  => "A standard template with name '$Param{Name}' already exists!"
+            Message  => "A standard template with the name '$Param{Name}' already exists.",
         );
         return;
     }
@@ -311,7 +305,7 @@ sub StandardTemplateUpdate {
     return 1;
 }
 
-=item StandardTemplateLookup()
+=head2 StandardTemplateLookup()
 
 return the name or the standard template id
 
@@ -389,11 +383,11 @@ sub StandardTemplateLookup {
     return $Self->{"StandardTemplate$Suffix"};
 }
 
-=item StandardTemplateList()
+=head2 StandardTemplateList()
 
 get all valid standard templates
 
-    my %StandardTemplates = $StandardTemplatesObject->StandardTemplateList();
+    my %StandardTemplates = $StandardTemplateObject->StandardTemplateList();
 
 Returns:
     %StandardTemplates = (
@@ -472,7 +466,7 @@ sub StandardTemplateList {
     return %Data;
 }
 
-=item NameExistsCheck()
+=head2 NameExistsCheck()
 
     return 1 if another standard template with this name already exists
 
@@ -509,14 +503,12 @@ sub NameExistsCheck {
 
 1;
 
-=back
-
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<https://otrs.org/>).
+This software is part of the OTRS project (L<http://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (GPL). If you
-did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
+the enclosed file COPYING for license information (AGPL). If you
+did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
